@@ -37,7 +37,7 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/postmaster/bgwriter.c,v 1.13.4.1 2005/02/19 23:16:27 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/postmaster/bgwriter.c,v 1.21.2.1 2005/12/08 19:19:31 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -51,6 +51,7 @@
 #include "miscadmin.h"
 #include "postmaster/bgwriter.h"
 #include "storage/bufmgr.h"
+#include "storage/fd.h"
 #include "storage/freespace.h"
 #include "storage/ipc.h"
 #include "storage/pmsignal.h"
@@ -58,6 +59,7 @@
 #include "tcop/tcopprot.h"
 #include "utils/guc.h"
 #include "utils/memutils.h"
+#include "utils/resowner.h"
 
 
 /*----------
@@ -86,8 +88,14 @@
  *	6. If ckpt_failed is different from the originally saved value,
  *	   assume request failed; otherwise it was definitely successful.
  *
+ * An additional field is ckpt_time_warn; this is also sig_atomic_t for
+ * simplicity, but is only used as a boolean.  If a backend is requesting
+ * a checkpoint for which a checkpoints-too-close-together warning is
+ * reasonable, it should set this field TRUE just before sending the signal.
+ *
  * The requests array holds fsync requests sent by backends and not yet
- * absorbed by the bgwriter.
+ * absorbed by the bgwriter.  Unlike the checkpoint fields, the requests
+ * fields are protected by BgWriterCommLock.
  *----------
  */
 typedef struct
@@ -105,6 +113,8 @@ typedef struct
 	sig_atomic_t ckpt_done;		/* advances when checkpoint done */
 	sig_atomic_t ckpt_failed;	/* advances when checkpoint fails */
 
+	sig_atomic_t ckpt_time_warn;	/* warn if too soon since last ckpt? */
+
 	int			num_requests;	/* current # of requests */
 	int			max_requests;	/* allocated array size */
 	BgWriterRequest requests[1];	/* VARIABLE LENGTH ARRAY */
@@ -116,9 +126,6 @@ static BgWriterShmemStruct *BgWriterShmem;
  * GUC parameters
  */
 int			BgWriterDelay = 200;
-int			BgWriterPercent = 1;
-int			BgWriterMaxPages = 100;
-
 int			CheckPointTimeout = 300;
 int			CheckPointWarning = 30;
 
@@ -155,6 +162,7 @@ void
 BackgroundWriterMain(void)
 {
 	sigjmp_buf	local_sigjmp_buf;
+	MemoryContext bgwriter_context;
 
 	Assert(BgWriterShmem != NULL);
 	BgWriterShmem->bgwriter_pid = MyProcPid;
@@ -165,11 +173,11 @@ BackgroundWriterMain(void)
 	 *
 	 * Note: we deliberately ignore SIGTERM, because during a standard Unix
 	 * system shutdown cycle, init will SIGTERM all processes at once.	We
-	 * want to wait for the backends to exit, whereupon the postmaster
-	 * will tell us it's okay to shut down (via SIGUSR2).
+	 * want to wait for the backends to exit, whereupon the postmaster will
+	 * tell us it's okay to shut down (via SIGUSR2).
 	 *
-	 * SIGUSR1 is presently unused; keep it spare in case someday we want
-	 * this process to participate in sinval messaging.
+	 * SIGUSR1 is presently unused; keep it spare in case someday we want this
+	 * process to participate in sinval messaging.
 	 */
 	pqsignal(SIGHUP, BgSigHupHandler);	/* set flag to read config file */
 	pqsignal(SIGINT, ReqCheckpointHandler);		/* request checkpoint */
@@ -197,11 +205,30 @@ BackgroundWriterMain(void)
 #endif
 
 	/*
-	 * Initialize so that first time-driven checkpoint happens at the
-	 * correct time.
+	 * Initialize so that first time-driven checkpoint happens at the correct
+	 * time.
 	 */
 	last_checkpoint_time = time(NULL);
 
+	/*
+	 * Create a resource owner to keep track of our resources (currently
+	 * only buffer pins).
+	 */
+	CurrentResourceOwner = ResourceOwnerCreate(NULL, "Background Writer");
+
+	/*
+	 * Create a memory context that we will do all our work in.  We do this so
+	 * that we can reset the context during error recovery and thereby avoid
+	 * possible memory leaks.  Formerly this code just ran in
+	 * TopMemoryContext, but resetting that would be a really bad idea.
+	 */
+	bgwriter_context = AllocSetContextCreate(TopMemoryContext,
+											 "Background Writer",
+											 ALLOCSET_DEFAULT_MINSIZE,
+											 ALLOCSET_DEFAULT_INITSIZE,
+											 ALLOCSET_DEFAULT_MAXSIZE);
+	MemoryContextSwitchTo(bgwriter_context);
+
 	/*
 	 * If an exception is encountered, processing resumes here.
 	 *
@@ -221,11 +248,18 @@ BackgroundWriterMain(void)
 		/*
 		 * These operations are really just a minimal subset of
 		 * AbortTransaction().	We don't have very many resources to worry
-		 * about in bgwriter, but we do have LWLocks and buffers.
+		 * about in bgwriter, but we do have LWLocks, buffers, and temp files.
 		 */
 		LWLockReleaseAll();
 		AbortBufferIO();
 		UnlockBuffers();
+		/* buffer pins are released here: */
+		ResourceOwnerRelease(CurrentResourceOwner,
+							 RESOURCE_RELEASE_BEFORE_LOCKS,
+							 false, true);
+		/* we needn't bother with the other ResourceOwnerRelease phases */
+		AtEOXact_Buffers(false);
+		AtEOXact_Files();
 
 		/* Warn any waiting backends that the checkpoint failed. */
 		if (ckpt_active)
@@ -239,20 +273,22 @@ BackgroundWriterMain(void)
 		}
 
 		/*
-		 * Now return to normal top-level context and clear ErrorContext
-		 * for next time.
+		 * Now return to normal top-level context and clear ErrorContext for
+		 * next time.
 		 */
-		MemoryContextSwitchTo(TopMemoryContext);
+		MemoryContextSwitchTo(bgwriter_context);
 		FlushErrorState();
 
+		/* Flush any leaked data in the top-level context */
+		MemoryContextResetAndDeleteChildren(bgwriter_context);
+
 		/* Now we can allow interrupts again */
 		RESUME_INTERRUPTS();
 
 		/*
-		 * Sleep at least 1 second after any error.  A write error is
-		 * likely to be repeated, and we don't want to be filling the
-		 * error logs as fast as we can.  (XXX think about ways to make
-		 * progress when the LRU dirty buffer cannot be written...)
+		 * Sleep at least 1 second after any error.  A write error is likely
+		 * to be repeated, and we don't want to be filling the error logs as
+		 * fast as we can.
 		 */
 		pg_usleep(1000000L);
 	}
@@ -274,7 +310,6 @@ BackgroundWriterMain(void)
 		bool		force_checkpoint = false;
 		time_t		now;
 		int			elapsed_secs;
-		int			n;
 		long		udelay;
 
 		/*
@@ -309,8 +344,8 @@ BackgroundWriterMain(void)
 		}
 
 		/*
-		 * Do an unforced checkpoint if too much time has elapsed since
-		 * the last one.
+		 * Do an unforced checkpoint if too much time has elapsed since the
+		 * last one.
 		 */
 		now = time(NULL);
 		elapsed_secs = now - last_checkpoint_time;
@@ -323,20 +358,20 @@ BackgroundWriterMain(void)
 		 */
 		if (do_checkpoint)
 		{
-			if (CheckPointWarning != 0)
-			{
-				/*
-				 * Ideally we should only warn if this checkpoint was
-				 * requested due to running out of segment files, and not
-				 * if it was manually requested.  However we can't tell
-				 * the difference with the current signalling mechanism.
-				 */
-				if (elapsed_secs < CheckPointWarning)
-					ereport(LOG,
-							(errmsg("checkpoints are occurring too frequently (%d seconds apart)",
-									elapsed_secs),
-							 errhint("Consider increasing the configuration parameter \"checkpoint_segments\".")));
-			}
+			/*
+			 * We will warn if (a) too soon since last checkpoint (whatever
+			 * caused it) and (b) somebody has set the ckpt_time_warn flag
+			 * since the last checkpoint start.  Note in particular that this
+			 * implementation will not generate warnings caused by
+			 * CheckPointTimeout < CheckPointWarning.
+			 */
+			if (BgWriterShmem->ckpt_time_warn &&
+				elapsed_secs < CheckPointWarning)
+				ereport(LOG,
+						(errmsg("checkpoints are occurring too frequently (%d seconds apart)",
+								elapsed_secs),
+						 errhint("Consider increasing the configuration parameter \"checkpoint_segments\".")));
+			BgWriterShmem->ckpt_time_warn = false;
 
 			/*
 			 * Indicate checkpoint start to any waiting backends.
@@ -348,8 +383,7 @@ BackgroundWriterMain(void)
 
 			/*
 			 * After any checkpoint, close all smgr files.	This is so we
-			 * won't hang onto smgr references to deleted files
-			 * indefinitely.
+			 * won't hang onto smgr references to deleted files indefinitely.
 			 */
 			smgrcloseall();
 
@@ -361,29 +395,30 @@ BackgroundWriterMain(void)
 
 			/*
 			 * Note we record the checkpoint start time not end time as
-			 * last_checkpoint_time.  This is so that time-driven
-			 * checkpoints happen at a predictable spacing.
+			 * last_checkpoint_time.  This is so that time-driven checkpoints
+			 * happen at a predictable spacing.
 			 */
 			last_checkpoint_time = now;
-
-			/* Nap for configured time before rechecking */
-			n = 1;
 		}
 		else
-			n = BufferSync(BgWriterPercent, BgWriterMaxPages);
+			BgBufferSync();
 
 		/*
-		 * Nap for the configured time or sleep for 10 seconds if there
-		 * was nothing to do at all.
+		 * Nap for the configured time, or sleep for 10 seconds if there is no
+		 * bgwriter activity configured.
 		 *
-		 * On some platforms, signals won't interrupt the sleep.  To ensure
-		 * we respond reasonably promptly when someone signals us, break
-		 * down the sleep into 1-second increments, and check for
-		 * interrupts after each nap.
+		 * On some platforms, signals won't interrupt the sleep.  To ensure we
+		 * respond reasonably promptly when someone signals us, break down the
+		 * sleep into 1-second increments, and check for interrupts after each
+		 * nap.
 		 *
 		 * We absorb pending requests after each short sleep.
 		 */
-		udelay = ((n > 0) ? BgWriterDelay : 10000) * 1000L;
+		if ((bgwriter_all_percent > 0.0 && bgwriter_all_maxpages > 0) ||
+			(bgwriter_lru_percent > 0.0 && bgwriter_lru_maxpages > 0))
+			udelay = BgWriterDelay * 1000L;
+		else
+			udelay = 10000000L;
 		while (udelay > 1000000L)
 		{
 			if (got_SIGHUP || checkpoint_requested || shutdown_requested)
@@ -416,13 +451,13 @@ bg_quickdie(SIGNAL_ARGS)
 
 	/*
 	 * DO NOT proc_exit() -- we're here because shared memory may be
-	 * corrupted, so we don't want to try to clean up our transaction.
-	 * Just nail the windows shut and get out of town.
+	 * corrupted, so we don't want to try to clean up our transaction. Just
+	 * nail the windows shut and get out of town.
 	 *
-	 * Note we do exit(1) not exit(0).	This is to force the postmaster into
-	 * a system reset cycle if some idiot DBA sends a manual SIGQUIT to a
-	 * random backend.	This is necessary precisely because we don't clean
-	 * up our shared memory state.
+	 * Note we do exit(1) not exit(0).	This is to force the postmaster into a
+	 * system reset cycle if some idiot DBA sends a manual SIGQUIT to a random
+	 * backend.  This is necessary precisely because we don't clean up our
+	 * shared memory state.
 	 */
 	exit(1);
 }
@@ -458,15 +493,19 @@ ReqShutdownHandler(SIGNAL_ARGS)
  * BgWriterShmemSize
  *		Compute space needed for bgwriter-related shared memory
  */
-int
+Size
 BgWriterShmemSize(void)
 {
+	Size		size;
+
 	/*
-	 * Currently, the size of the requests[] array is arbitrarily set
-	 * equal to NBuffers.  This may prove too large or small ...
+	 * Currently, the size of the requests[] array is arbitrarily set equal to
+	 * NBuffers.  This may prove too large or small ...
 	 */
-	return MAXALIGN(sizeof(BgWriterShmemStruct) +
-					(NBuffers - 1) *sizeof(BgWriterRequest));
+	size = offsetof(BgWriterShmemStruct, requests);
+	size = add_size(size, mul_size(NBuffers, sizeof(BgWriterRequest)));
+
+	return size;
 }
 
 /*
@@ -500,9 +539,13 @@ BgWriterShmemInit(void)
  * If waitforit is true, wait until the checkpoint is completed
  * before returning; otherwise, just signal the request and return
  * immediately.
+ *
+ * If warnontime is true, and it's "too soon" since the last checkpoint,
+ * the bgwriter will log a warning.  This should be true only for checkpoints
+ * caused due to xlog filling, else the warning will be misleading.
  */
 void
-RequestCheckpoint(bool waitforit)
+RequestCheckpoint(bool waitforit, bool warnontime)
 {
 	/* use volatile pointer to prevent code rearrangement */
 	volatile BgWriterShmemStruct *bgs = BgWriterShmem;
@@ -517,15 +560,18 @@ RequestCheckpoint(bool waitforit)
 		CreateCheckPoint(false, true);
 
 		/*
-		 * After any checkpoint, close all smgr files.	This is so we
-		 * won't hang onto smgr references to deleted files
-		 * indefinitely.
+		 * After any checkpoint, close all smgr files.	This is so we won't
+		 * hang onto smgr references to deleted files indefinitely.
 		 */
 		smgrcloseall();
 
 		return;
 	}
 
+	/* Set warning request flag if appropriate */
+	if (warnontime)
+		bgs->ckpt_time_warn = true;
+
 	/*
 	 * Send signal to request checkpoint.  When waitforit is false, we
 	 * consider failure to send the signal to be nonfatal.
@@ -538,8 +584,8 @@ RequestCheckpoint(bool waitforit)
 			 "could not signal for checkpoint: %m");
 
 	/*
-	 * If requested, wait for completion.  We detect completion according
-	 * to the algorithm given above.
+	 * If requested, wait for completion.  We detect completion according to
+	 * the algorithm given above.
 	 */
 	if (waitforit)
 	{
@@ -552,13 +598,12 @@ RequestCheckpoint(bool waitforit)
 
 		/*
 		 * We are waiting for ckpt_done >= old_started, in a modulo sense.
-		 * This is a little tricky since we don't know the width or
-		 * signedness of sig_atomic_t.	We make the lowest common
-		 * denominator assumption that it is only as wide as "char".  This
-		 * means that this algorithm will cope correctly as long as we
-		 * don't sleep for more than 127 completed checkpoints.  (If we
-		 * do, we will get another chance to exit after 128 more
-		 * checkpoints...)
+		 * This is a little tricky since we don't know the width or signedness
+		 * of sig_atomic_t.  We make the lowest common denominator assumption
+		 * that it is only as wide as "char".  This means that this algorithm
+		 * will cope correctly as long as we don't sleep for more than 127
+		 * completed checkpoints.  (If we do, we will get another chance to
+		 * exit after 128 more checkpoints...)
 		 */
 		while (((signed char) (bgs->ckpt_done - old_started)) < 0)
 		{
@@ -619,7 +664,7 @@ ForwardFsyncRequest(RelFileNode rnode, BlockNumber segno)
  *
  * This is exported because it must be called during CreateCheckPoint;
  * we have to be sure we have accepted all pending requests *after* we
- * establish the checkpoint redo pointer.  Since CreateCheckPoint
+ * establish the checkpoint REDO pointer.  Since CreateCheckPoint
  * sometimes runs in non-bgwriter processes, do nothing if not bgwriter.
  */
 void
@@ -633,8 +678,17 @@ AbsorbFsyncRequests(void)
 		return;
 
 	/*
-	 * We try to avoid holding the lock for a long time by copying the
-	 * request array.
+	 * We have to PANIC if we fail to absorb all the pending requests (eg,
+	 * because our hashtable runs out of memory).  This is because the system
+	 * cannot run safely if we are unable to fsync what we have been told to
+	 * fsync.  Fortunately, the hashtable is so small that the problem is
+	 * quite unlikely to arise in practice.
+	 */
+	START_CRIT_SECTION();
+
+	/*
+	 * We try to avoid holding the lock for a long time by copying the request
+	 * array.
 	 */
 	LWLockAcquire(BgWriterCommLock, LW_EXCLUSIVE);
 
@@ -650,6 +704,9 @@ AbsorbFsyncRequests(void)
 
 	for (request = requests; n > 0; request++, n--)
 		RememberFsyncRequest(request->rnode, request->segno);
+
 	if (requests)
 		pfree(requests);
+
+	END_CRIT_SECTION();
 }
