@@ -1,7 +1,7 @@
 /*
  * conversion functions between pg_wchar and multibyte streams.
  * Tatsuo Ishii
- * $PostgreSQL: pgsql/src/backend/utils/mb/wchar.c,v 1.40 2004/12/03 01:20:20 momjian Exp $
+ * $PostgreSQL: pgsql/src/backend/utils/mb/wchar.c,v 1.47.2.1 2005/12/24 09:42:30 ishii Exp $
  *
  * WIN1250 client encoding updated by Pavel Behal
  *
@@ -58,7 +58,6 @@ pg_ascii_dsplen(const unsigned char *s)
 /*
  * EUC
  */
-
 static int	pg_euc2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
 {
@@ -66,26 +65,26 @@ static int	pg_euc2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 2)
+		if (*from == SS2 && len >= 2)	/* JIS X 0201 (so called "1 byte KANA") */
 		{
 			from++;
-			*to = 0xff & *from++;
+			*to = (SS2 << 8) | *from++;
 			len -= 2;
 		}
-		else if (*from == SS3 && len >= 3)
+		else if (*from == SS3 && len >= 3)		/* JIS X 0212 KANJI */
 		{
 			from++;
-			*to = *from++ << 8;
-			*to |= 0x3f & *from++;
+			*to = (SS3 << 16) | (*from++ << 8);
+			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)
+		else if ((*from & 0x80) && len >= 2)	/* JIS X 0208 KANJI */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
 			len -= 2;
 		}
-		else
+		else	/* must be ASCII */
 		{
 			*to = *from++;
 			len--;
@@ -183,6 +182,7 @@ pg_euckr_dsplen(const unsigned char *s)
 
 /*
  * EUC_CN
+ *
  */
 static int	pg_euccn2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
@@ -191,21 +191,21 @@ static int	pg_euccn2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 3)
+		if (*from == SS2 && len >= 3)	/* code set 2 (unused?) */
 		{
 			from++;
-			*to = 0x3f00 & (*from++ << 8);
-			*to = *from++;
+			*to = (SS2 << 16) | (*from++ << 8);
+			*to |= *from++;
 			len -= 3;
 		}
-		else if (*from == SS3 && len >= 3)
+		else if (*from == SS3 && len >= 3)		/* code set 3 (unsed ?) */
 		{
 			from++;
-			*to = *from++ << 8;
-			*to |= 0x3f & *from++;
+			*to = (SS3 << 16) | (*from++ << 8);
+			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)
+		else if ((*from & 0x80) && len >= 2)	/* code set 1 */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -249,6 +249,7 @@ pg_euccn_dsplen(const unsigned char *s)
 
 /*
  * EUC_TW
+ *
  */
 static int	pg_euctw2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
@@ -257,22 +258,22 @@ static int	pg_euctw2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 4)
+		if (*from == SS2 && len >= 4)	/* code set 2 */
 		{
 			from++;
-			*to = *from++ << 16;
+			*to = (SS2 << 24) | (*from++ << 16) ;
 			*to |= *from++ << 8;
 			*to |= *from++;
 			len -= 4;
 		}
-		else if (*from == SS3 && len >= 3)
+		else if (*from == SS3 && len >= 3)		/* code set 3 (unused?) */
 		{
 			from++;
-			*to = *from++ << 8;
-			*to |= 0x3f & *from++;
+			*to = (SS3 << 16) | (*from++ << 8);
+			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)
+		else if ((*from & 0x80) && len >= 2)	/* code set 2 */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -344,7 +345,7 @@ pg_johab_dsplen(const unsigned char *s)
 }
 
 /*
- * convert UTF-8 string to pg_wchar (UCS-2)
+ * convert UTF8 string to pg_wchar (UCS-2)
  * caller should allocate enough space for "to"
  * len: length of from.
  * "from" not necessarily null terminated.
@@ -395,7 +396,7 @@ pg_utf2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 }
 
 /*
- * returns the byte length of a UTF-8 word pointed to by s
+ * returns the byte length of a UTF8 word pointed to by s
  */
 int
 pg_utf_mblen(const unsigned char *s)
@@ -406,8 +407,14 @@ pg_utf_mblen(const unsigned char *s)
 		len = 1;
 	else if ((*s & 0xe0) == 0xc0)
 		len = 2;
-	else if ((*s & 0xe0) == 0xe0)
+	else if ((*s & 0xf0) == 0xe0)
 		len = 3;
+	else if ((*s & 0xf8) == 0xf0)
+		len = 4;
+	else if ((*s & 0xfc) == 0xf8)
+		len = 5;
+	else if ((*s & 0xfe) == 0xfc)
+		len = 6;
 	return (len);
 }
 
@@ -721,7 +728,7 @@ pg_wchar_tbl pg_wchar_table[] = {
 	{pg_euckr2wchar_with_len, pg_euckr_mblen, pg_euckr_dsplen, 3},		/* 3; PG_EUC_KR */
 	{pg_euctw2wchar_with_len, pg_euctw_mblen, pg_euctw_dsplen, 3},		/* 4; PG_EUC_TW */
 	{pg_johab2wchar_with_len, pg_johab_mblen, pg_johab_dsplen, 3},		/* 5; PG_JOHAB */
-	{pg_utf2wchar_with_len, pg_utf_mblen, pg_utf_dsplen, 3},	/* 6; PG_UNICODE */
+	{pg_utf2wchar_with_len, pg_utf_mblen, pg_utf_dsplen, 4},	/* 6; PG_UTF8 */
 	{pg_mule2wchar_with_len, pg_mule_mblen, pg_mule_dsplen, 3}, /* 7; PG_MULE_INTERNAL */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 8; PG_LATIN1 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 9; PG_LATIN2 */
@@ -734,11 +741,12 @@ pg_wchar_tbl pg_wchar_table[] = {
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 16; PG_LATIN9 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 17; PG_LATIN10 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 18; PG_WIN1256 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 19; PG_TCVN */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 19; PG_WIN1258 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 20; PG_WIN874 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 21; PG_KOI8 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 22; PG_WIN1251 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 23; PG_ALT */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 22; PG_WIN1252 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 23; PG_WIN866 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 24; ISO-8859-5 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 25; ISO-8859-6 */
 	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 26; ISO-8859-7 */
@@ -762,28 +770,28 @@ pg_mic_mblen(const unsigned char *mbstr)
  * Returns the byte length of a multibyte word.
  */
 int
-pg_encoding_mblen(int encoding, const unsigned char *mbstr)
+pg_encoding_mblen(int encoding, const char *mbstr)
 {
 	Assert(PG_VALID_ENCODING(encoding));
 
 	return ((encoding >= 0 &&
 			 encoding < sizeof(pg_wchar_table) / sizeof(pg_wchar_tbl)) ?
-			((*pg_wchar_table[encoding].mblen) (mbstr)) :
-			((*pg_wchar_table[PG_SQL_ASCII].mblen) (mbstr)));
+		((*pg_wchar_table[encoding].mblen) ((const unsigned char *) mbstr)) :
+	((*pg_wchar_table[PG_SQL_ASCII].mblen) ((const unsigned char *) mbstr)));
 }
 
 /*
  * Returns the display length of a multibyte word.
  */
 int
-pg_encoding_dsplen(int encoding, const unsigned char *mbstr)
+pg_encoding_dsplen(int encoding, const char *mbstr)
 {
 	Assert(PG_VALID_ENCODING(encoding));
 
 	return ((encoding >= 0 &&
 			 encoding < sizeof(pg_wchar_table) / sizeof(pg_wchar_tbl)) ?
-			((*pg_wchar_table[encoding].dsplen) (mbstr)) :
-			((*pg_wchar_table[PG_SQL_ASCII].dsplen) (mbstr)));
+	   ((*pg_wchar_table[encoding].dsplen) ((const unsigned char *) mbstr)) :
+	((*pg_wchar_table[PG_SQL_ASCII].dsplen) ((const unsigned char *) mbstr)));
 }
 
 /*
@@ -799,6 +807,60 @@ pg_encoding_max_length(int encoding)
 
 #ifndef FRONTEND
 
+bool
+pg_utf8_islegal(const unsigned char *source, int length)
+{
+	unsigned char a;
+	const unsigned char *srcptr = source + length;
+
+	switch (length)
+	{
+		default:
+			return false;
+			/* Everything else falls through when "true"... */
+		case 4:
+			if ((a = (*--srcptr)) < 0x80 || a > 0xBF)
+				return false;
+		case 3:
+			if ((a = (*--srcptr)) < 0x80 || a > 0xBF)
+				return false;
+		case 2:
+			if ((a = (*--srcptr)) > 0xBF)
+				return false;
+			switch (*source)
+			{
+					/* no fall-through in this inner switch */
+				case 0xE0:
+					if (a < 0xA0)
+						return false;
+					break;
+				case 0xED:
+					if (a > 0x9F)
+						return false;
+					break;
+				case 0xF0:
+					if (a < 0x90)
+						return false;
+					break;
+				case 0xF4:
+					if (a > 0x8F)
+						return false;
+					break;
+				default:
+					if (a < 0x80)
+						return false;
+			}
+
+		case 1:
+			if (*source >= 0x80 && *source < 0xC2)
+				return false;
+	}
+	if (*source > 0xF4)
+		return false;
+	return true;
+}
+
+
 /*
  * Verify mbstr to make sure that it has a valid character sequence.
  * mbstr is not necessarily NULL terminated; length of mbstr is
@@ -808,7 +870,7 @@ pg_encoding_max_length(int encoding)
  * true; when noError is false, ereport() a descriptive message.
  */
 bool
-pg_verifymbstr(const unsigned char *mbstr, int len, bool noError)
+pg_verifymbstr(const char *mbstr, int len, bool noError)
 {
 	int			l;
 	int			i;
@@ -822,51 +884,55 @@ pg_verifymbstr(const unsigned char *mbstr, int len, bool noError)
 
 	while (len > 0 && *mbstr)
 	{
-		/* special UTF-8 check */
-		if (encoding == PG_UTF8 && (*mbstr & 0xf8) == 0xf0)
-		{
-			if (noError)
-				return false;
-			ereport(ERROR,
-					(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
-					 errmsg("Unicode characters greater than or equal to 0x10000 are not supported")));
-		}
-
 		l = pg_mblen(mbstr);
 
-		for (i = 1; i < l; i++)
+		/* special UTF-8 check */
+		if (encoding == PG_UTF8)
 		{
-			/*
-			 * we expect that every multibyte char consists of bytes
-			 * having the 8th bit set
-			 */
-			if (i >= len || (mbstr[i] & 0x80) == 0)
+			if (!pg_utf8_islegal((const unsigned char *) mbstr, l))
 			{
-				char		buf[8 * 2 + 1];
-				char	   *p = buf;
-				int			j,
-							jlimit;
-
 				if (noError)
 					return false;
-
-				jlimit = Min(l, len);
-				jlimit = Min(jlimit, 8);		/* prevent buffer overrun */
-
-				for (j = 0; j < jlimit; j++)
-					p += sprintf(p, "%02x", mbstr[j]);
-
 				ereport(ERROR,
 						(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
-				errmsg("invalid byte sequence for encoding \"%s\": 0x%s",
-					   GetDatabaseEncodingName(), buf)));
+						 errmsg("invalid UTF-8 byte sequence detected near byte 0x%02x",
+								(unsigned char) *mbstr)));
 			}
 		}
+		else
+		{
+			for (i = 1; i < l; i++)
+			{
+				/*
+				 * we expect that every multibyte char consists of bytes
+				 * having the 8th bit set
+				 */
+				if (i >= len || (mbstr[i] & 0x80) == 0)
+				{
+					char		buf[8 * 2 + 1];
+					char	   *p = buf;
+					int			j,
+								jlimit;
 
+					if (noError)
+						return false;
+
+					jlimit = Min(l, len);
+					jlimit = Min(jlimit, 8);	/* prevent buffer overrun */
+
+					for (j = 0; j < jlimit; j++)
+						p += sprintf(p, "%02x", (unsigned char) mbstr[j]);
+
+					ereport(ERROR,
+							(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
+					errmsg("invalid byte sequence for encoding \"%s\": 0x%s",
+						   GetDatabaseEncodingName(), buf)));
+				}
+			}
+		}
 		len -= l;
 		mbstr += l;
 	}
-
 	return true;
 }
 
