@@ -4,10 +4,10 @@
  *	  Declarations for routines exported from lexer and parser files.
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: gramparse.h,v 1.25.2.1 2006/05/21 20:12:20 tgl Exp $
+ * $PostgreSQL: pgsql/src/include/parser/gramparse.h,v 1.31 2004/12/31 22:03:38 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -15,27 +15,14 @@
 #ifndef GRAMPARSE_H
 #define GRAMPARSE_H
 
-#include "lib/stringinfo.h"
 #include "nodes/parsenodes.h"
 
-typedef enum
-{
-	BACKSLASH_QUOTE_OFF,
-	BACKSLASH_QUOTE_ON,
-	BACKSLASH_QUOTE_SAFE_ENCODING
-} BackslashQuoteType;
-
-/* GUC variables in scan.l (every one of these is a bad idea :-() */
-extern BackslashQuoteType backslash_quote;
-
 
 /* from parser.c */
-extern void parser_param_set(Oid *typev, int nargs);
-extern Oid	param_type(int t);
 extern int	yylex(void);
 
 /* from scan.l */
-extern void scanner_init(StringInfo str);
+extern void scanner_init(const char *str);
 extern void scanner_finish(void);
 extern int	base_yylex(void);
 extern void yyerror(const char *message);
